part of datepicker;

class _MonthViewMultiSelectionPainter extends _IMonthViewPainter {
  _MonthViewMultiSelectionPainter(
      this.visibleDates,
      this.rowCount,
      this.cellStyle,
      this.selectionTextStyle,
      this.rangeTextStyle,
      this.selectionColor,
      this.startRangeSelectionColor,
      this.endRangeSelectionColor,
      this.rangeSelectionColor,
      this.datePickerTheme,
      this.isRtl,
      this.todayHighlightColor,
      this.minDate,
      this.maxDate,
      this.enablePastDates,
      this.showLeadingAndTailingDates,
      this.blackoutDates,
      this.specialDates,
      this.weekendDays,
      this.selectedDates,
      this.selectionShape,
      this.selectionRadius,
      this._mouseHoverPosition,
      this.enableMultiView,
      this.multiViewSpacing,
      this.selectionNotifier)
      : super(selectionNotifier: selectionNotifier);

  final int rowCount;
  final DateRangePickerMonthCellStyle cellStyle;
  final List<DateTime> visibleDates;
  final bool isRtl;
  final Color todayHighlightColor;
  final SfDateRangePickerThemeData datePickerTheme;
  final DateTime minDate;
  final DateTime maxDate;
  final bool enablePastDates;
  final bool showLeadingAndTailingDates;
  final List<DateTime> blackoutDates;
  final List<DateTime> specialDates;
  final List<int> weekendDays;
  final DateRangePickerSelectionShape selectionShape;
  final double selectionRadius;
  final bool enableMultiView;
  final double multiViewSpacing;
  final Offset _mouseHoverPosition;
  final TextStyle selectionTextStyle;
  final TextStyle rangeTextStyle;
  final Color selectionColor;
  final Color startRangeSelectionColor;
  final Color endRangeSelectionColor;
  final Color rangeSelectionColor;
  List<DateTime> selectedDates;
  ValueNotifier<bool> selectionNotifier;
  Paint _selectionPainter;
  TextPainter _textPainter;
  double _cellWidth, _cellHeight;
  double _centerXPosition, _centerYPosition;

  @override
  TextStyle _drawSelection(Canvas canvas, double x, double y, int index,
      TextStyle selectionTextStyle, TextStyle selectionRangeTextStyle) {
    _selectionPainter.isAntiAlias = true;
    if (selectionShape == DateRangePickerSelectionShape.circle) {
      final double _radius =
          _getCellRadius(selectionRadius, _centerXPosition, _centerYPosition);
      _drawCircleSelection(canvas, x + _centerXPosition, y + _centerYPosition,
          _radius, _selectionPainter);
    } else {
      _drawFillSelection(
          canvas, x, y, _cellWidth, _cellHeight, _selectionPainter);
    }

    return selectionTextStyle;
  }

  @override
  List<int> _getSelectedIndexValues(int viewStartIndex, int viewEndIndex) {
    final List<int> _selectedIndex = <int>[];
    if (selectedDates != null) {
      for (int j = 0; j < selectedDates.length; j++) {
        final DateTime date = selectedDates[j];
        if (!isDateWithInDateRange(
            visibleDates[viewStartIndex], visibleDates[viewEndIndex], date)) {
          continue;
        }

        _selectedIndex.add(_getSelectedIndex(date, visibleDates,
            viewStartIndex: viewStartIndex));
      }
    }

    return _selectedIndex;
  }

  @override
  void paint(Canvas canvas, Size size) {
    _cellWidth = size.width / _IMonthViewPainter._numberOfDaysInWeek;
    if (enableMultiView) {
      _cellWidth = (size.width - multiViewSpacing) /
          (_IMonthViewPainter._numberOfDaysInWeek * 2);
    }

    _cellHeight = size.height / rowCount;
    _centerXPosition = _cellWidth / 2;
    _centerYPosition = _cellHeight / 2;
    _drawMonthCellsAndSelection(canvas, size, this, _cellWidth, _cellHeight);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _MonthViewMultiSelectionPainter oldWidget = oldDelegate;
    return oldWidget.visibleDates != visibleDates ||
        oldWidget.rowCount != rowCount ||
        oldWidget.todayHighlightColor != todayHighlightColor ||
        oldWidget.enablePastDates != enablePastDates ||
        oldWidget.showLeadingAndTailingDates != showLeadingAndTailingDates ||
        oldWidget.cellStyle != cellStyle ||
        oldWidget.minDate != minDate ||
        oldWidget.maxDate != maxDate ||
        oldWidget.enableMultiView != enableMultiView ||
        oldWidget.multiViewSpacing != multiViewSpacing ||
        oldWidget.selectedDates != selectedDates ||
        oldWidget.blackoutDates != blackoutDates ||
        oldWidget.specialDates != specialDates ||
        oldWidget.selectionShape != selectionShape ||
        oldWidget.selectionRadius != selectionRadius ||
        oldWidget.rangeSelectionColor != rangeSelectionColor ||
        oldWidget.endRangeSelectionColor != endRangeSelectionColor ||
        oldWidget.startRangeSelectionColor != startRangeSelectionColor ||
        oldWidget.selectionColor != selectionColor ||
        oldWidget.rangeTextStyle != rangeTextStyle ||
        oldWidget.selectionTextStyle != selectionTextStyle ||
        oldWidget.isRtl != isRtl ||
        oldWidget.datePickerTheme != datePickerTheme ||
        (kIsWeb && oldWidget._mouseHoverPosition != _mouseHoverPosition);
  }

  @override
  void _updateSelection(_PickerStateArgs details) {
    if (_isDateCollectionEquals(details._selectedDates, selectedDates)) {
      return;
    }

    selectedDates = _cloneList(details._selectedDates);
    selectionNotifier.value = !selectionNotifier.value;
  }

  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      return _getSemanticsBuilderForMonthView(size, rowCount, this);
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    final _MonthViewMultiSelectionPainter _oldWidget = oldDelegate;
    return _oldWidget.visibleDates != visibleDates;
  }
}
