part of datepicker;

class _PickerCenturyViewPainter extends _IPickerYearView {
  _PickerCenturyViewPainter(
      this.visibleDates,
      this.cellStyle,
      this.minDate,
      this.maxDate,
      this.enablePastDates,
      this.todayHighlightColor,
      this.selectionShape,
      this.isRtl,
      this.datePickerTheme,
      this._locale,
      this._mouseHoverPosition,
      this.enableMultiView,
      this.multiViewSpacing,
      this.selectionTextStyle,
      this.rangeTextStyle,
      this.selectionColor,
      this.startRangeSelectionColor,
      this.endRangeSelectionColor,
      this.rangeSelectionColor,
      this.selectedDate,
      this._selectionMode,
      this._selectionRadius,
      this._selectionNotifier)
      : super(selectionNotifier: _selectionNotifier);

  final DateRangePickerYearCellStyle cellStyle;
  final List<DateTime> visibleDates;
  final bool isRtl;
  final Color todayHighlightColor;
  final DateTime minDate;
  final DateTime maxDate;
  final bool enablePastDates;
  final DateRangePickerSelectionShape selectionShape;
  final Offset _mouseHoverPosition;
  final DateRangePickerSelectionMode _selectionMode;
  final double _selectionRadius;
  final bool enableMultiView;
  final double multiViewSpacing;
  final ValueNotifier<bool> _selectionNotifier;
  final TextStyle selectionTextStyle;
  final TextStyle rangeTextStyle;
  final Color selectionColor;
  final Color startRangeSelectionColor;
  final Color endRangeSelectionColor;
  final Color rangeSelectionColor;
  dynamic selectedDate;
  TextPainter _textPainter;
  final Locale _locale;
  Paint _todayHighlightPaint;
  final SfDateRangePickerThemeData datePickerTheme;

  @override
  void paint(Canvas canvas, Size size) {
    _drawYearCells(canvas, size, this, DateRangePickerView.century);
  }

  @override
  bool _isSameView(DateTime date, DateTime currentDate) {
    if (date == null || currentDate == null) {
      return false;
    }

    if (date.year ~/ 10 == currentDate.year ~/ 10) {
      return true;
    }

    return false;
  }

  @override
  bool _isTrailingDate(int index, int viewStartIndex) {
    final DateTime _currentDate = visibleDates[index];
    final DateTime _viewStartDate = visibleDates[viewStartIndex];
    return _currentDate.year ~/ 100 != _viewStartDate.year ~/ 100;
  }

  @override
  bool _isBetweenMinMaxMonth(DateTime date) {
    if (date == null || minDate == null || maxDate == null) {
      return true;
    }

    final int today = DateTime.now().year ~/ 10;
    final int currentYear = date.year ~/ 10;
    if (currentYear >= (minDate.year ~/ 10) &&
        currentYear <= (maxDate.year ~/ 10) &&
        (enablePastDates || (!enablePastDates && currentYear >= today))) {
      return true;
    }

    return false;
  }

  @override
  void _updateSelection(_PickerStateArgs details) {
    _updateYearViewSelection(this, details);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _PickerCenturyViewPainter oldWidget = oldDelegate;
    return oldWidget.visibleDates != visibleDates ||
        oldWidget.todayHighlightColor != todayHighlightColor ||
        oldWidget.enablePastDates != enablePastDates ||
        oldWidget.cellStyle != cellStyle ||
        oldWidget.minDate != minDate ||
        oldWidget.maxDate != maxDate ||
        oldWidget.selectionShape != selectionShape ||
        oldWidget.isRtl != isRtl ||
        oldWidget.datePickerTheme != datePickerTheme ||
        oldWidget.enableMultiView != enableMultiView ||
        oldWidget.multiViewSpacing != multiViewSpacing ||
        oldWidget.selectedDate != selectedDate ||
        oldWidget._selectionMode != _selectionMode ||
        oldWidget.rangeSelectionColor != rangeSelectionColor ||
        oldWidget.endRangeSelectionColor != endRangeSelectionColor ||
        oldWidget.startRangeSelectionColor != startRangeSelectionColor ||
        oldWidget.selectionColor != selectionColor ||
        oldWidget.rangeTextStyle != rangeTextStyle ||
        oldWidget.selectionTextStyle != selectionTextStyle ||
        oldWidget._selectionRadius != _selectionRadius ||
        oldWidget._locale != _locale ||
        (kIsWeb && oldWidget._mouseHoverPosition != _mouseHoverPosition);
  }

  @override
  String _getCellText(DateTime date) {
    return date.year.toString() + ' - ' + (date.year + 9).toString();
  }

  @override
  bool _isCurrentView(DateTime date, int index) {
    final int datesCount =
        enableMultiView ? visibleDates.length ~/ 2 : visibleDates.length;
    final int middleIndex = (index * datesCount) + (datesCount ~/ 2);
    final int currentYear = visibleDates[middleIndex].year;
    return currentYear ~/ 100 == date.year ~/ 100;
  }

  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      return _getSemanticsBuilderForYearView(size, this);
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    final _PickerCenturyViewPainter _oldWidget = oldDelegate;
    return _oldWidget.visibleDates != visibleDates;
  }
}
