part of datepicker;

class _PickerYearViewPainter extends _IPickerYearView {
  _PickerYearViewPainter(
      this.visibleDates,
      this.cellStyle,
      this.minDate,
      this.maxDate,
      this.enablePastDates,
      this.todayHighlightColor,
      this.selectionShape,
      this.monthFormat,
      this.isRtl,
      this.datePickerTheme,
      this._locale,
      this._mouseHoverPosition,
      this.enableMultiView,
      this.multiViewSpacing,
      this.selectionTextStyle,
      this.rangeTextStyle,
      this.selectionColor,
      this.startRangeSelectionColor,
      this.endRangeSelectionColor,
      this.rangeSelectionColor,
      this.selectedDate,
      this._selectionMode,
      this._selectionRadius,
      this._selectionNotifier)
      : super(selectionNotifier: _selectionNotifier);

  final DateRangePickerYearCellStyle cellStyle;
  final List<DateTime> visibleDates;
  final bool isRtl;
  final String monthFormat;
  final Color todayHighlightColor;
  final DateTime minDate;
  final DateTime maxDate;
  final bool enablePastDates;
  final DateRangePickerSelectionShape selectionShape;
  final Locale _locale;
  final Offset _mouseHoverPosition;
  final DateRangePickerSelectionMode _selectionMode;
  final double _selectionRadius;
  final bool enableMultiView;
  final double multiViewSpacing;
  final ValueNotifier<bool> _selectionNotifier;
  final TextStyle selectionTextStyle;
  final TextStyle rangeTextStyle;
  final Color selectionColor;
  final Color startRangeSelectionColor;
  final Color endRangeSelectionColor;
  final Color rangeSelectionColor;
  dynamic selectedDate;
  TextPainter _textPainter;
  Paint _todayHighlightPaint;
  final SfDateRangePickerThemeData datePickerTheme;

  @override
  void paint(Canvas canvas, Size size) {
    _drawYearCells(canvas, size, this, DateRangePickerView.year);
  }

  @override
  bool _isSameView(DateTime date, DateTime currentDate) {
    if (date == null || currentDate == null) {
      return false;
    }

    if (date.month == currentDate.month && date.year == currentDate.year) {
      return true;
    }

    return false;
  }

  @override
  bool _isTrailingDate(int index, int viewStartIndex) {
    return false;
  }

  @override
  bool _isBetweenMinMaxMonth(DateTime date) {
    if (date == null || minDate == null || maxDate == null) {
      return true;
    }

    final DateTime today = DateTime.now();
    if (((date.month >= minDate.month && date.year == minDate.year) ||
            date.year > minDate.year) &&
        ((date.month <= maxDate.month && date.year == maxDate.year) ||
            date.year < maxDate.year) &&
        (enablePastDates ||
            (!enablePastDates &&
                ((date.month >= today.month && date.year == today.year) ||
                    date.year > today.year)))) {
      return true;
    }

    return false;
  }

  @override
  void _updateSelection(_PickerStateArgs details) {
    _updateYearViewSelection(this, details);
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) {
    final _PickerYearViewPainter oldWidget = oldDelegate;
    return oldWidget.visibleDates != visibleDates ||
        oldWidget.todayHighlightColor != todayHighlightColor ||
        oldWidget.enablePastDates != enablePastDates ||
        oldWidget.cellStyle != cellStyle ||
        oldWidget.minDate != minDate ||
        oldWidget.maxDate != maxDate ||
        oldWidget.enableMultiView != enableMultiView ||
        oldWidget.multiViewSpacing != multiViewSpacing ||
        oldWidget.selectionShape != selectionShape ||
        oldWidget.monthFormat != monthFormat ||
        oldWidget.isRtl != isRtl ||
        oldWidget.datePickerTheme != datePickerTheme ||
        oldWidget.selectedDate != selectedDate ||
        oldWidget._selectionMode != _selectionMode ||
        oldWidget.rangeSelectionColor != rangeSelectionColor ||
        oldWidget.endRangeSelectionColor != endRangeSelectionColor ||
        oldWidget.startRangeSelectionColor != startRangeSelectionColor ||
        oldWidget.selectionColor != selectionColor ||
        oldWidget.rangeTextStyle != rangeTextStyle ||
        oldWidget.selectionTextStyle != selectionTextStyle ||
        oldWidget._selectionRadius != _selectionRadius ||
        oldWidget._locale != _locale ||
        (kIsWeb && oldWidget._mouseHoverPosition != _mouseHoverPosition);
  }

  @override
  String _getCellText(DateTime date) {
    return DateFormat(
            monthFormat == null || monthFormat.isEmpty ? 'MMM' : monthFormat,
            _locale.toString())
        .format(date)
        .toString();
  }

  @override
  bool _isCurrentView(DateTime date, int index) {
    return true;
  }

  @override
  SemanticsBuilderCallback get semanticsBuilder {
    return (Size size) {
      return _getSemanticsBuilderForYearView(size, this);
    };
  }

  @override
  bool shouldRebuildSemantics(CustomPainter oldDelegate) {
    final _PickerYearViewPainter _oldWidget = oldDelegate;
    return _oldWidget.visibleDates != visibleDates;
  }
}
